"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.destroy = exports.create = void 0;
const sequelize_1 = require("sequelize");
const actions_1 = __importStar(require("@nocobase/actions"));
function create(context, next) {
    return __awaiter(this, void 0, void 0, function* () {
        return actions_1.default.create(context, () => __awaiter(this, void 0, void 0, function* () {
            const { body: instance, db } = context;
            const repository = actions_1.utils.getRepositoryFromParams(context);
            if (!instance.upstreamId) {
                const previousHead = yield repository.findOne({
                    filter: {
                        id: {
                            $ne: instance.id
                        },
                        upstreamId: null
                    }
                });
                if (previousHead) {
                    yield previousHead.setUpstream(instance);
                    yield instance.setDownstream(previousHead);
                    instance.set('downstream', previousHead);
                }
                return next();
            }
            const upstream = yield instance.getUpstream();
            if (instance.branchIndex == null) {
                const downstream = yield upstream.getDownstream();
                if (downstream) {
                    yield downstream.setUpstream(instance);
                    yield instance.setDownstream(downstream);
                    instance.set('downstream', downstream);
                }
                yield upstream.update({
                    downstreamId: instance.id
                });
                upstream.set('downstream', instance);
            }
            else {
                const [downstream] = yield upstream.getBranches({
                    where: {
                        id: {
                            [sequelize_1.Op.ne]: instance.id
                        },
                        branchIndex: instance.branchIndex
                    }
                });
                if (downstream) {
                    yield downstream.update({
                        upstreamId: instance.id,
                        branchIndex: null
                    });
                    yield instance.setDownstream(downstream);
                    instance.set('downstream', downstream);
                }
            }
            instance.set('upstream', upstream);
            yield next();
        }));
    });
}
exports.create = create;
function searchBranchNodes(nodes, from) {
    const branchHeads = nodes
        .filter((item) => item.upstreamId === from.id && item.branchIndex != null);
    return branchHeads.reduce((flatten, head) => flatten.concat(searchBranchDownstreams(nodes, head)), []);
}
function searchBranchDownstreams(nodes, from) {
    let result = [];
    for (let search = from; search; search = search.downstream) {
        result = [...result, search, ...searchBranchNodes(nodes, search)];
    }
    return result;
}
function destroy(context, next) {
    return __awaiter(this, void 0, void 0, function* () {
        const repository = actions_1.utils.getRepositoryFromParams(context);
        const { db } = context;
        const { filterByTk } = context.action.params;
        context.body = yield db.sequelize.transaction((transaction) => __awaiter(this, void 0, void 0, function* () {
            const fields = ['id', 'upstreamId', 'downstreamId', 'branchIndex'];
            const instance = yield repository.findOne({
                filterByTk,
                fields: [...fields, 'workflowId'],
                appends: ['upstream', 'downstream'],
                transaction
            });
            const { upstream, downstream } = instance.get();
            if (upstream && upstream.downstreamId === instance.id) {
                yield upstream.update({
                    downstreamId: instance.downstreamId
                }, { transaction });
            }
            if (downstream) {
                yield downstream.update({
                    upstreamId: instance.upstreamId,
                    branchIndex: instance.branchIndex
                }, { transaction });
            }
            const nodes = yield repository.find({
                filter: {
                    workflowId: instance.workflowId
                },
                fields,
                transaction
            });
            const nodesMap = new Map();
            // make map
            nodes.forEach(item => {
                nodesMap.set(item.id, item);
            });
            // overwrite
            nodesMap.set(instance.id, instance);
            // make linked list
            nodes.forEach(item => {
                if (item.upstreamId) {
                    item.upstream = nodesMap.get(item.upstreamId);
                }
                if (item.downstreamId) {
                    item.downstream = nodesMap.get(item.downstreamId);
                }
            });
            const branchNodes = searchBranchNodes(nodes, instance);
            yield repository.destroy({
                filterByTk: [instance.id, ...branchNodes.map(item => item.id)],
                transaction
            });
            return instance;
        }));
        yield next();
    });
}
exports.destroy = destroy;
//# sourceMappingURL=flow_nodes.js.map