"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.calculate = exports.calculators = void 0;
const lodash_1 = require("lodash");
const utils_1 = require("@nocobase/utils");
exports.calculators = new utils_1.Registry();
exports.default = exports.calculators;
// @deprecated
// HACK: if no path provided, return self
// @see https://github.com/lodash/lodash/pull/1270
// TODO(question): should add default value as lodash?
function get(object, path) {
    return path == null || !path.length ? object : (0, lodash_1.get)(object, path);
}
// NOTE:
//  this method could only be used in executing nodes.
//  because type of 'job' need loaded jobs in runtime execution.
//  or the execution should be prepared first.
function calculate(operand, lastJob, execution) {
    switch (operand.type) {
        // @Deprecated
        // from execution context
        case '$context':
            return get(execution.context, operand.options.path);
        // @Deprecated
        // from last job (or input job)
        case '$input':
            return lastJob !== null && lastJob !== void 0 ? lastJob : get(lastJob.result, operand.options.path);
        // @Deprecated
        // from job in execution
        case '$jobsMapByNodeId':
            // assume jobs have been fetched from execution before
            const job = execution.jobsMapByNodeId[operand.options.nodeId];
            return job && get(job, operand.options.path);
        case '$calculation':
            const fn = exports.calculators.get(operand.options.calculator);
            if (!fn) {
                throw new Error(`no calculator function registered for "${operand.options.calculator}"`);
            }
            return fn(...operand.options.operands.map(item => calculate(item, lastJob, execution)));
        // constant
        default:
            return operand.value;
    }
}
exports.calculate = calculate;
// built-in functions
function equal(a, b) {
    return a === b;
}
function notEqual(a, b) {
    return a !== b;
}
function gt(a, b) {
    return a > b;
}
function gte(a, b) {
    return a >= b;
}
function lt(a, b) {
    return a < b;
}
function lte(a, b) {
    return a <= b;
}
exports.calculators.register('equal', equal);
exports.calculators.register('notEqual', notEqual);
exports.calculators.register('gt', gt);
exports.calculators.register('gte', gte);
exports.calculators.register('lt', lt);
exports.calculators.register('lte', lte);
exports.calculators.register('===', equal);
exports.calculators.register('!==', notEqual);
exports.calculators.register('>', gt);
exports.calculators.register('>=', gte);
exports.calculators.register('<', lt);
exports.calculators.register('<=', lte);
function add(...args) {
    return args.reduce((sum, a) => sum + a, 0);
}
function minus(a, b) {
    return a - b;
}
function multiple(...args) {
    return args.reduce((result, a) => result * a, 1);
}
function divide(a, b) {
    return a / b;
}
function mod(a, b) {
    return a % b;
}
exports.calculators.register('add', add);
exports.calculators.register('minus', minus);
exports.calculators.register('multiple', multiple);
exports.calculators.register('divide', divide);
exports.calculators.register('mod', mod);
exports.calculators.register('+', add);
exports.calculators.register('-', minus);
exports.calculators.register('*', multiple);
exports.calculators.register('/', divide);
exports.calculators.register('%', mod);
function includes(a, b) {
    return a.includes(b);
}
function notIncludes(a, b) {
    return !a.includes(b);
}
function startsWith(a, b) {
    return a.startsWith(b);
}
function notStartsWith(a, b) {
    return !a.startsWith(b);
}
function endsWith(a, b) {
    return a.endsWith(b);
}
function notEndsWith(a, b) {
    return !a.endsWith(b);
}
exports.calculators.register('includes', includes);
exports.calculators.register('notIncludes', notIncludes);
exports.calculators.register('startsWith', startsWith);
exports.calculators.register('notStartsWith', notStartsWith);
exports.calculators.register('endsWith', endsWith);
exports.calculators.register('notEndsWith', notEndsWith);
function before(a, b) {
    return a < b;
}
exports.calculators.register('now', () => new Date());
// TODO: add more common calculators
//# sourceMappingURL=index.js.map