"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PARALLEL_MODE = void 0;
const constants_1 = require("../constants");
exports.PARALLEL_MODE = {
    ALL: 'all',
    ANY: 'any',
    RACE: 'race'
};
const StatusGetters = {
    [exports.PARALLEL_MODE.ALL](result) {
        if (result.some(j => j && j.status === constants_1.JOB_STATUS.REJECTED)) {
            return constants_1.JOB_STATUS.REJECTED;
        }
        if (result.every(j => j && j.status === constants_1.JOB_STATUS.RESOLVED)) {
            return constants_1.JOB_STATUS.RESOLVED;
        }
        return constants_1.JOB_STATUS.PENDING;
    },
    [exports.PARALLEL_MODE.ANY](result) {
        return result.some(j => j && j.status === constants_1.JOB_STATUS.RESOLVED)
            ? constants_1.JOB_STATUS.RESOLVED
            : (result.some(j => j && j.status === constants_1.JOB_STATUS.PENDING)
                ? constants_1.JOB_STATUS.PENDING
                : constants_1.JOB_STATUS.REJECTED);
    },
    [exports.PARALLEL_MODE.RACE](result) {
        return result.some(j => j && j.status === constants_1.JOB_STATUS.RESOLVED)
            ? constants_1.JOB_STATUS.RESOLVED
            : (result.some(j => j && j.status === constants_1.JOB_STATUS.REJECTED)
                ? constants_1.JOB_STATUS.REJECTED
                : constants_1.JOB_STATUS.PENDING);
    }
};
exports.default = {
    run(prevJob, execution) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const branches = execution.nodes
                .filter(item => item.upstream === this && item.branchIndex !== null)
                .sort((a, b) => a.branchIndex - b.branchIndex);
            const job = yield execution.saveJob({
                status: constants_1.JOB_STATUS.PENDING,
                result: Array(branches.length).fill(null),
                nodeId: this.id,
                upstreamId: (_a = prevJob === null || prevJob === void 0 ? void 0 : prevJob.id) !== null && _a !== void 0 ? _a : null
            });
            // NOTE:
            // use `reduce` but not `Promise.all` here to avoid racing manupulating db.
            // for users, this is almost equivalent to `Promise.all`,
            // because of the delay is not significant sensible.
            // another better aspect of this is, it could handle sequenced branches in future.
            yield branches.reduce((promise, branch) => promise.then(() => execution.run(branch, job)), Promise.resolve());
            return execution.end(this, job);
        });
    },
    resume(branchJob, execution) {
        return __awaiter(this, void 0, void 0, function* () {
            const job = execution.findBranchParentJob(branchJob, this);
            const { result, status } = job;
            // if parallel has been done (resolved / rejected), do not care newly executed branch jobs.
            if (status !== constants_1.JOB_STATUS.PENDING) {
                return null;
            }
            // find the index of the node which start the branch
            const jobNode = execution.nodesMap.get(branchJob.nodeId);
            const { branchIndex } = execution.findBranchStartNode(jobNode);
            const { mode = exports.PARALLEL_MODE.ALL } = this.config || {};
            const newResult = [...result.slice(0, branchIndex), branchJob.get(), ...result.slice(branchIndex + 1)];
            job.set({
                result: newResult,
                status: StatusGetters[mode](newResult)
            });
            if (job.status === constants_1.JOB_STATUS.PENDING) {
                yield job.save({ transaction: execution.transaction });
                return execution.end(this, job);
            }
            return job;
        });
    }
};
//# sourceMappingURL=parallel.js.map