"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const database_1 = require("@nocobase/database");
const triggers_1 = __importDefault(require("../triggers"));
const constants_1 = require("../constants");
class WorkflowModel extends database_1.Model {
    static mount() {
        return __awaiter(this, void 0, void 0, function* () {
            const collection = this.database.getCollection('workflows');
            const workflows = yield collection.repository.find({
                filter: { enabled: true },
            });
            workflows.forEach((workflow) => {
                workflow.toggle();
            });
            this.addHook('afterCreate', (model) => model.toggle());
            this.addHook('afterUpdate', (model) => model.toggle());
            this.addHook('afterDestroy', (model) => model.toggle(false));
        });
    }
    getHookId() {
        return `workflow-${this.get('id')}`;
    }
    getTransaction(options) {
        if (!this.useTransaction) {
            return undefined;
        }
        return options.transaction && !options.transaction.finished
            ? options.transaction
            : this.constructor.database.sequelize.transaction();
    }
    toggle(enable) {
        return __awaiter(this, void 0, void 0, function* () {
            const type = this.get('type');
            const { on, off } = triggers_1.default.get(type);
            if (typeof enable !== 'undefined' ? enable : this.get('enabled')) {
                on.call(this, this.trigger.bind(this));
            }
            else {
                off.call(this);
            }
        });
    }
    trigger(context, options) {
        return __awaiter(this, void 0, void 0, function* () {
            // `null` means not to trigger
            if (context === null) {
                return;
            }
            const transaction = yield this.getTransaction(options);
            if (this.useTransaction) {
                const existed = yield this.countExecutions({
                    where: {
                        transaction: transaction.id
                    },
                    transaction
                });
                if (existed) {
                    console.warn(`workflow ${this.id} has already been triggered in same execution (${transaction.id}), and newly triggering will be skipped.`);
                    return;
                }
            }
            const execution = yield this.createExecution({
                context,
                status: constants_1.EXECUTION_STATUS.STARTED,
                useTransaction: this.useTransaction,
                transaction: transaction.id
            }, { transaction });
            execution.workflow = this;
            yield execution.start({ transaction });
            if (transaction && (!options.transaction || options.transaction.finished)) {
                yield transaction.commit();
            }
            return execution;
        });
    }
}
exports.default = WorkflowModel;
//# sourceMappingURL=Workflow.js.map